package com.mydomain;

import java.io.*;
import lejos.nxt.Button;
import lejos.nxt.LCD;
import lejos.nxt.Motor;
import lejos.nxt.comm.BTConnection;
import lejos.nxt.comm.Bluetooth;
import lejos.util.Delay;

//Bluetooth thread, continuously checks for incoming connections.
public class BTReceive extends Thread
{
	public void run()
	{
		while(true){
			try
			{
				BTConnection connection = Bluetooth.waitForConnection();
				DataInputStream input = connection.openDataInputStream();
				DataOutputStream output = connection.openDataOutputStream();
										
				switch (input.readInt()) {
			
				//Location requested, respond with robotname and current location.
				case 0:
					int busy;
					if(NXTSwarm.state == 1 || NXTSwarm.state == 2){
						busy = 0;
					}
					else{
						busy = 1;
					}
					output.writeChar(Bluetooth.getFriendlyName().charAt(0));
					output.writeFloat(NXTSwarm.location.getX());
					output.writeFloat(NXTSwarm.location.getY());
					output.writeFloat(NXTSwarm.location.getF());
					output.writeInt(busy);
					output.flush();
					input.close();
					output.close();
					connection.close();
				break;
			
				//Help requested, receive robot location and calculate direction and distance.
				case 1:
				
					NXTSwarm.helpneeded = true;
					synchronized(this){
						wait();
					}
					NXTSwarm.helpRobot = String.valueOf(input.readChar());
				
					//Receive robot location.
					float xhelp = input.readFloat();
					float yhelp = input.readFloat();
					float fhelp = input.readFloat();
				
				
					//Calculate position behind robot.
					xhelp -= 20*Math.cos(fhelp);
					yhelp -= 20*Math.sin(fhelp);

					float Fx = (float)Math.cos(NXTSwarm.location.getF());
					float Fy = (float)Math.sin(NXTSwarm.location.getF());
							
					float Fhelpx = (float)Math.cos(xhelp);
					float Fhelpy = (float)Math.sin(yhelp);
				
					//Calculate vector towards other robot.
					NXTSwarm.x =  xhelp - NXTSwarm.location.getX();
					NXTSwarm.y  = yhelp - NXTSwarm.location.getY();
				
					//Calculate angle to turn towards other robot.
					NXTSwarm.angle = (float)Math.acos((Fx*NXTSwarm.x + Fy*NXTSwarm.y)/
					(Math.sqrt(Math.pow(NXTSwarm.x, 2)+Math.pow(NXTSwarm.y, 2))));
					//Store other robot angle.
					NXTSwarm.otherRobotAngle = fhelp;
					//Calculate angle relative to X axis.
					NXTSwarm.AngleRelativeVectorToXAxis  = (float)Math.atan(NXTSwarm.y/NXTSwarm.x);
				
					if(NXTSwarm.y>0 && NXTSwarm.x<0){
							NXTSwarm.AngleRelativeVectorToXAxis = 
							(float)Math.PI - NXTSwarm.AngleRelativeVectorToXAxis;
					}
					
					if(NXTSwarm.y<0 && NXTSwarm.x>0){
							NXTSwarm.AngleRelativeVectorToXAxis = 
							(float)Math.PI*2 - NXTSwarm.AngleRelativeVectorToXAxis;
					}
					if(NXTSwarm.y<0 && NXTSwarm.x<0){
							NXTSwarm.AngleRelativeVectorToXAxis = 
							(float)Math.PI + NXTSwarm.AngleRelativeVectorToXAxis;
					}
				
					//Calculate direction to turn.			
					if(NXTSwarm.AngleRelativeVectorToXAxis  > NXTSwarm.location.getF()   && 
					NXTSwarm.AngleRelativeVectorToXAxis < (NXTSwarm.location.getF()+Math.PI) && 
					NXTSwarm.location.getF() < Math.PI ){
						NXTSwarm.angle = -NXTSwarm.angle;
					}
					if((NXTSwarm.AngleRelativeVectorToXAxis > NXTSwarm.location.getF() || 
					NXTSwarm.AngleRelativeVectorToXAxis < NXTSwarm.location.getF()-Math.PI) && 
					NXTSwarm.location.getF() > Math.PI ){
						NXTSwarm.angle = -NXTSwarm.angle;
					}
					//Notify NXTSwarm that the angle and distance is calculated.
					synchronized(NXTSwarm){
						NXTSwarm.notify();
					}					
					input.close();
					output.close();
					connection.close();			
					break;
				
				//Other robot has arrived to help.
				case 2:
					NXTSwarm.state = 6;
					input.close();
					output.close();
					connection.close();			
					break;
				}

			}
			catch(Exception ioe)
			{	LCD.clear();
				LCD.drawString("ERROR", 0, 0);
				LCD.drawString(ioe.getMessage(), 2, 0);
				LCD.refresh();
			}
		}
	}

}
