package com.mydomain;

import java.io.*;
import javax.bluetooth.RemoteDevice;
import lejos.nxt.Button;
import lejos.nxt.LCD;
import lejos.nxt.comm.BTConnection;
import lejos.nxt.comm.Bluetooth;
import lejos.util.Delay;

//Bluetooth send class, provides methods to send request over bluetooth.
public class BTSend
{
	public void askLocations() 
	{
		//Ask the location off every robot, except itself.
		for (String name : new String[]{"A","B","C"}){
			if(!name.equals(Bluetooth.getFriendlyName())){
				try{
					RemoteDevice receiver = Bluetooth.getKnownDevice(name);
					if (receiver == null){
						throw new IOException("no such" + name + " " +Bluetooth.getFriendlyName());
					}
					BTConnection connection = Bluetooth.connect(receiver);
					if (connection == null){
						throw new IOException("Connect fail");
					}
					DataInputStream input = connection.openDataInputStream();
					DataOutputStream output = connection.openDataOutputStream();

					output.writeInt(0);
					output.flush();
					
					//Recieve robot name and location.
					char robotid = input.readChar();
					float x = input.readFloat();
					float y = input.readFloat();
					float f = input.readFloat();
					int busy = input.readInt();

					//Store robot name and location.
					NXTSwarm.robotLocations.put(String.valueOf(robotid), new float[] {x,y,f, busy});
					
					input.close();
					output.close();
					connection.close();
				}
				catch(Exception ioe)
				{
					LCD.clear();
					LCD.drawString("ERROR", 0, 0);
					LCD.drawString(ioe.getMessage(), 2, 0);
					LCD.refresh();
				}
			}
		}
	}
	
	public void help(String name) 
	{
		try
		{	
			//Ask other robot for help.
			RemoteDevice receiver = Bluetooth.getKnownDevice(name);
			if (receiver == null){
				throw new IOException("no asking" + name);
			}
			BTConnection connection = Bluetooth.connect(receiver);
			if (connection == null){
				throw new IOException("Connect fail");
			}
			DataInputStream input = connection.openDataInputStream();
			DataOutputStream output = connection.openDataOutputStream();

			output.writeInt(1);
			//Send name and location to other robot.
			output.writeChar(Bluetooth.getFriendlyName().charAt(0));
			output.writeFloat(NXTSwarm.location.getX());
			output.writeFloat(NXTSwarm.location.getY());
			output.writeFloat(NXTSwarm.location.getF());

			output.flush();
			input.close();
			output.close();
			connection.close();
		}
		catch(Exception ioe)
		{
			LCD.clear();
			LCD.drawString("ERROR", 0, 0);
			LCD.drawString(ioe.getMessage(), 2, 0);
			LCD.refresh();
		}
	}
	
	public void arrived(String name) 
	{
		try
		{
			//Let robot know you have arrived to help.
			RemoteDevice receiver = Bluetooth.getKnownDevice(name);
			if (receiver == null){
				throw new IOException("no such, arrived");
			}
			BTConnection connection = Bluetooth.connect(receiver);
			if (connection == null){
				throw new IOException("Connect fail");
			}
			LCD.drawString("start sending arrived", 0, 2);
			DataInputStream input = connection.openDataInputStream();
			DataOutputStream output = connection.openDataOutputStream();			
			
			output.writeInt(2);
			output.flush();
			input.close();
			output.close();
			connection.close();			
		}
		catch(Exception ioe)
		{
			LCD.clear();
			LCD.drawString("ERROR", 0, 0);
			LCD.drawString(ioe.getMessage(), 2, 0);
			LCD.refresh();
		}
		
	}

}


