package com.mydomain;

import lejos.util.Delay;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import lejos.nxt.*;


public class NXTSwarm {

	public static boolean   helpneeded = false; //True if other robot needs help, false otherwise.
	public static float angle = 0;
	public static float otherRobotAngle = 0;
	public static String helpRobot = "";
	public static float x = 0;
	public static float y = 0;
	public static float AngleRelativeVectorToXAxis;
	public static int speed    = 1000;    //Motor speed
	public static int distance = 40;  //Threshold for ultrasonic sensor.
	public static int lighttreshhold    = 425; //Threshold for light sensor.
	public static int state    = 1; //Set initial state.
	public static Location  location = new Location(-100,100,315); //Initial location.
	public static Map<String,float[]> robotLocations = new HashMap<>(); //Hashmap to store locations of other robots when asked.

	public static void main(String[] args) {

		//Set variables for sensor usage
		UltrasonicSensor sonic     =   new UltrasonicSensor(SensorPort.S2);
		TouchSensor touchleft      =   new TouchSensor(SensorPort.S1);
		TouchSensor touchright     =   new TouchSensor(SensorPort.S4);
		LightSensor light          =   new LightSensor(SensorPort.S3);
		BTSend btSend              =   new BTSend();
		BTReceive btReceive        =   new BTReceive();

		// Start bluetooth receive thread.
		btReceive.start();
		//Start infinite loop.
		while(true){
			//Stop program when ESCAPE button is pressed.
			if(Button.ESCAPE.isDown()){
				btReceive.interrupt();
				break;
			}

			switch (state) {

				case 1: //Searching straight ahead
                    //Move forwards.
					straight(false);
					while (true)  {
                        //Check if an object is within threshold distance. If so stop moving and go to state 3.
						if 	(sonic.getDistance()<distance) {
							stop();
							state = 3; 
							break;
						}
						//Check if the edge of the field of operation is reached. If so stop moving and go to state 2.
						if(light.getNormalizedLightValue()<lighttreshhold) {
							stop();
							state = 2;
							break;
						}		
						//If another robots request help, stop moving and go to state 5.
						if(helpneeded) {
							stop();
							state = 5;
							break;
						}
					}
				break;

				case 2: //Out of bounds, move backwards and turn then go back to state 1.
					straight(true, 30);
					turn(120);
					state = 1;
				break;

				case 3: //Move towards object
					straight(false);
					while(true) {
						//Check if the edge of the field of operation is reached. If so, the object is out of bounds, stop moving and go to state 2.
						if(light.getNormalizedLightValue()<lighttreshhold){
							stop();
							state = 2;
							break;
						}
						//Check if front bumper is pressed. If so, the object is too heavy and help is needed. Stop moving and go to state 4.
						if (touchleft.isPressed() || touchright.isPressed()){
							stop();
							state = 4;
							break;
						}						
					}
				break;

				case 4: //Determine which robot is closest
					btSend.askLocations();
					float minDistance = 10000;
					float distance = 0;
					float[] loc;
					String minRobot   = "";
					//Calculate distance to robot A.
					if(robotLocations.containsKey("A")){
						loc = robotLocations.get("A");
						distance = (float) Math.sqrt((Math.pow(loc[0]-location.getX(), 2))+(Math.pow(loc[1]-location.getY(), 2)));
						if(distance < minDistance && loc[3] == 0){
							minDistance = distance;
							minRobot = "A";
						}
					}
					//Calculate distance to robot B.
					if(robotLocations.containsKey("B")){
						loc = robotLocations.get("B");
						distance = (float) Math.sqrt((Math.pow(loc[0]-location.getX(), 2))+(Math.pow(loc[1]-location.getY(), 2)));
						if(distance < minDistance && loc[3] == 0){
							minDistance = distance;
							minRobot = "B";
						}
					}
					//Calculate distance to robot C.
					if(robotLocations.containsKey("C")){
						loc = robotLocations.get("C");
						distance = (float) Math.sqrt((Math.pow(loc[0]-location.getX(), 2))+(Math.pow(loc[1]-location.getY(), 2)));
						if(distance < minDistance && loc[3] == 0){
							minDistance = distance;
							minRobot = "C";
						}
					}
					btSend.help(minRobot);
					robotLocations.clear();
					state = 7;
				break;

				case 5: //going to help
					//Notify bluetooth thread that it can start calculating the angle and distance.
					synchronized(btReceive){
						btReceive.notify();
					}
					//Wait for bluetooth thread to calculate angle and distance.
					synchronized(this){
						wait();
					}
					//Move towards other robot.
					turn((int)Math.toDegrees(angle));
					straight(false, ((int)Math.sqrt(Math.pow(x, 2)+Math.pow(y, 2))));
					turn((int)Math.toDegrees(-(newAngle - location.getF())));

					//Arrived at other robot, send message and go to state 7.
					btSend.arrived(helpRobot);
					helpneeded = false;
					state = 8;
				break;

				case 6:
					straight(false);
					while(true) {
						if(light.getNormalizedLightValue()<lighttreshhold){
							stop();
							state = 2;
							break;
						}
					}
				break;
				
				case: 7:
					//Waiting for other robot to help.
				break;

				case 8:
					//Helping other robot.
					straight(false);
					while(true) {
						//Check if edge of field of operation is reached or front bumper is pressed. If so, stop moving go to state 2.
						if((light.getNormalizedLightValue()<lighttreshhold) || touchleft.isPressed() || touchright.isPressed()){
							stop();
							state = 2;
							break;
						}
					}
				break;
			}
		}
	}

	public static void straight(boolean back){
		Motor.A.resetTachoCount();
		if(back){
			Motor.A.rotate(10000, true);
			Motor.C.rotate(10000, true);
		}else{
			Motor.A.rotate(-10000, true);
			Motor.C.rotate(-10000, true);
		}
	}
	
	//Move in the current direction of the robot and update the location.
	public static void straight(boolean back, int distance){
		Motor.A.resetTachoCount();
		if(back){
			Motor.A.rotate(distance*35, true);
			Motor.C.rotate(distance*35, false);
		}else{
			Motor.A.rotate(-distance*35, true);
			Motor.C.rotate(-distance*35, false);
		}
		location.updatestraight();
	}

	//Turn the robot and update the direction.
	public static void turn(int degrees){
		Motor.A.resetTachoCount();
		Motor.A.rotate(-degrees*(31/5), true);
		Motor.C.rotate(degrees*(31/5), false);
		location.updateturn(degrees);
	}

	//Stop all motors and update the position
	public static void stop(){
		Motor.A.stop(true);
		Motor.C.stop(true);
		location.updatestraight();
	}
}
